@@ -1,10 +1,7 @@
 from __future__ import annotations
 
 import asyncio
-
-# After we drop support for Python 3.10, we can use the built-in TimeoutError
-# instead of the one from asyncio since they are the same in Python 3.11+
-from asyncio import CancelledError, TimeoutError as asyncio_TimeoutError
+from asyncio import CancelledError
 from collections.abc import Callable
 from dataclasses import astuple, dataclass
 import enum
@@ -162,7 +159,7 @@ make_hello_request = _cached_make_hello_request
 def handle_timeout(fut: asyncio.Future[None]) -> None:
     """Handle a timeout."""
     if not fut.done():
-        fut.set_exception(asyncio_TimeoutError)
+        fut.set_exception(TimeoutError)
 
 
 _handle_timeout = handle_timeout
@@ -354,12 +351,12 @@ class APIConnection:
                         loop=self._loop,
                     )
                     break
-            except (OSError, asyncio_TimeoutError) as err:
+            except (OSError, TimeoutError) as err:
                 last_exception = err
                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
 
         if sock is None:
-            if isinstance(last_exception, asyncio_TimeoutError):
+            if isinstance(last_exception, TimeoutError):
                 raise TimeoutAPIError(
                     f"Timeout while connecting to {addrs}"
                 ) from last_exception
@@ -453,7 +450,7 @@ class APIConnection:
         )
         try:
             await self._frame_helper.ready_future
-        except asyncio_TimeoutError as err:
+        except TimeoutError as err:
             raise TimeoutAPIError(
                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
             ) from err
@@ -469,7 +466,12 @@ class APIConnection:
         msg_types = [HelloResponse]
         if login:
             messages.append(self._make_connect_request())
-            msg_types.append(ConnectResponse)
+            if self._params.password:
+                # Only wait for ConnectResponse if we actually have
+                # a password to send, but we will still register
+                # a handler for a ConnectResponse just in case
+                # the device has a password but we don't expect it
+                msg_types.append(ConnectResponse)
 
         responses = await self.send_messages_await_response_complex(
             tuple(messages),
@@ -479,9 +481,10 @@ class APIConnection:
             tuple(msg_types),
             CONNECT_REQUEST_TIMEOUT,
         )
+
         resp = responses.pop(0)
         self._process_hello_resp(resp)
-        if login:
+        if login and self._params.password:
             login_response = responses.pop(0)
             self._process_login_response(login_response)
 
@@ -490,6 +493,11 @@ class APIConnection:
         if login_response.invalid_password:
             raise InvalidAuthAPIError("Invalid password!")
 
+    def _handle_login_response(self, login_response: ConnectResponse) -> None:
+        """Handle a ConnectResponse."""
+        if login_response.invalid_password:
+            self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
+
     def _process_hello_resp(self, resp: HelloResponse) -> None:
         """Process a HelloResponse."""
         if self._debug_enabled:
@@ -605,8 +613,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("resolving", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("resolving", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_resolve_host_future()
         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
@@ -639,8 +648,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("starting", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("starting", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_start_connect_future()
         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)
@@ -709,8 +719,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("finishing", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("finishing", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_finish_connect_future()
         self._set_connection_state(CONNECTION_STATE_CONNECTED)
@@ -862,7 +873,7 @@ class APIConnection:
         timeout_expired = False
         try:
             await fut
-        except asyncio_TimeoutError as err:
+        except TimeoutError as err:
             timeout_expired = True
             response_names = message_types_to_names(msg_types)
             raise TimeoutAPIError(
@@ -904,7 +915,10 @@ class APIConnection:
                     "%s: Connection error occurred: %s",
                     self.log_name,
                     err or type(err),
-                    exc_info=not str(err),  # Log the full stack on empty error string
+                    exc_info=not str(err)
+                    or isinstance(
+                        err, UnhandledAPIConnectionError
+                    ),  # Log the full stack on empty error string
                 )
 
             # Only set the first error since otherwise the original
@@ -1012,6 +1026,9 @@ class APIConnection:
         self._add_message_callback_without_remove(
             self._handle_get_time_request_internal, (GetTimeRequest,)
         )
+        self._add_message_callback_without_remove(
+            self._handle_login_response, (ConnectResponse,)
+        )
 
     def _handle_disconnect_request_internal(  # pylint: disable=unused-argument
         self, _msg: DisconnectRequest
